import tailwind from "tailwindcss";
import postcss from "postcss";
import type { TailwindPluginOptions } from "./index";

export const { name: pkgName } = [require][0](
  process.env.BABEL_TAILWIND_BUILD ? "./package.json" : "../package.json"
);

interface LineColumn {
  line: number;
  column: number;
}

export interface SourceLocation {
  filename: string;
  start: LineColumn;
  end: LineColumn;
  text: string;
}

export interface StyleMapEntry {
  key: string;
  className: string;
  location: SourceLocation;
}

export const tailwindDirectives = ["components", "utilities", "variants"] as const;

export type StyleMap = Map</* filename */ string, StyleMapEntry[]>;

export function createPostCSS({
  tailwindConfig,
  postCSSPlugins = [],
  directives,
}: Pick<TailwindPluginOptions, "tailwindConfig" | "postCSSPlugins" | "directives">) {
  const post = postcss([
    tailwind({
      ...tailwindConfig,
      content: [{ raw: "<br>", extension: "html" }],
    }),
    ...postCSSPlugins,
  ]);

  const appliedDirectives = directives === "all" ? tailwindDirectives : directives;
  const directiveTexts = appliedDirectives?.map(d => `@tailwind ${d};\n`).join("") ?? "";

  return (css: string) => post.process(directiveTexts + css, { from: undefined });
}

export type Compile = ReturnType<typeof createPostCSS>;

export function toCSSText(tailwindMap: StyleMapEntry[]) {
  return tailwindMap
    .map(({ className, key }) => `.${key} {\n  @apply ${className}\n}`)
    .join("\n");
}
